@@ -31,3 +31,6 @@ CONF_NWK_UPDATE_ID_DEFAULT = 0x00
 CONF_OTA_IKEA_DEFAULT = False
 CONF_OTA_LEDVANCE_DEFAULT = False
 CONF_OTA_OTAU_DIR_DEFAULT = None
+CONF_TOPO_SCAN_PERIOD_DEFAULT = 240  # 4 hours
+CONF_TOPO_SCAN_ENABLED_DEFAULT = True
+CONF_TOPO_SKIP_COORDINATOR_DEFAULT = False
