@@ -1,11 +1,14 @@
 """Protocol Interfaces Functional Domain"""
 
+from __future__ import annotations
+
 import zigpy.types as t
 from zigpy.zcl import Cluster
 
 
 class DateTime(t.Struct):
-    _fields = [("date", t.uint32_t), ("time", t.uint32_t)]
+    date: t.uint32_t
+    time: t.uint32_t
 
 
 class GenericTunnel(Cluster):
@@ -37,7 +40,7 @@ class AnalogInputRegular(Cluster):
     attributes = {
         0x0016: ("cov_increment", t.Single),
         0x001F: ("device_type", t.CharacterString),
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x0076: ("update_interval", t.uint8_t),
@@ -79,7 +82,7 @@ class AnalogOutputRegular(Cluster):
     attributes = {
         0x0016: ("cov_increment", t.Single),
         0x001F: ("device_type", t.CharacterString),
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x0076: ("update_interval", t.uint8_t),
@@ -115,7 +118,7 @@ class AnalogValueRegular(Cluster):
     ep_attribute = "bacnet_regular_analog_value"
     attributes = {
         0x0016: ("cov_increment", t.Single),
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x00A8: ("profile_name", t.CharacterString),
@@ -153,7 +156,7 @@ class BinaryInputRegular(Cluster):
         0x0010: ("change_of_state_time", DateTime),
         0x001F: ("device_type", t.CharacterString),
         0x0021: ("elapsed_active_time", t.uint32_t),
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x0072: ("time_of_at_reset", DateTime),
@@ -191,7 +194,7 @@ class BinaryOutputRegular(Cluster):
         0x001F: ("device_type", t.CharacterString),
         0x0021: ("elapsed_active_time", t.uint32_t),
         0x0028: ("feed_back_value", t.enum8),
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x0072: ("time_of_at_reset", DateTime),
@@ -226,7 +229,7 @@ class BinaryValueRegular(Cluster):
         0x000F: ("change_of_state_count", t.uint32_t),
         0x0010: ("change_of_state_time", DateTime),
         0x0021: ("elapsed_active_time", t.uint32_t),
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x0072: ("time_of_at_reset", DateTime),
@@ -260,7 +263,7 @@ class MultistateInputRegular(Cluster):
     ep_attribute = "bacnet_regular_multistate_input"
     attributes = {
         0x001F: ("device_type", t.CharacterString),
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x00A8: ("profile_name", t.CharacterString),
@@ -294,7 +297,7 @@ class MultistateOutputRegular(Cluster):
     attributes = {
         0x001F: ("device_type", t.CharacterString),
         0x0028: ("feed_back_value", t.enum8),
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x00A8: ("profile_name", t.CharacterString),
@@ -324,7 +327,7 @@ class MultistateValueRegular(Cluster):
     cluster_id = 0x0612
     ep_attribute = "bacnet_regular_multistate_value"
     attributes = {
-        0x004B: ("object_id", t.fixed_list(4, t.uint8_t)),
+        0x004B: ("object_id", t.FixedList[4, t.uint8_t]),
         0x004D: ("object_name", t.CharacterString),
         0x004F: ("object_type", t.enum16),
         0x00A8: ("profile_name", t.CharacterString),
